// Copyright 2021 Intel Corporation
// SPDX-License-Identifier: Apache-2.0

#pragma once

#define OPENPGL_VERSION_MAJOR 0
#define OPENPGL_VERSION_MINOR 6
#define OPENPGL_VERSION_PATCH 0
#define OPENPGL_VERSION "0.6.0"
